@@ -17,20 +17,25 @@ with other dictionary (:func:`Dictionary.merge_with`) etc.
 
 from __future__ import with_statement
 
+from collections import Mapping
+import sys
 import logging
 import itertools
-import UserDict
 
 from gensim import utils
-from gensim._six import iteritems, iterkeys, itervalues, string_types
-from gensim._six.moves import xrange
-from gensim._six.moves import zip as izip
+
+if sys.version_info[0] >= 3:
+    unicode = str
+
+from six import PY3, iteritems, iterkeys, itervalues, string_types
+from six.moves import xrange
+from six.moves import zip as izip
 
 
 logger = logging.getLogger('gensim.corpora.dictionary')
 
 
-class Dictionary(utils.SaveLoad, UserDict.DictMixin):
+class Dictionary(utils.SaveLoad, Mapping):
     """
     Dictionary encapsulates the mapping between normalized words and their integer ids.
 
@@ -58,9 +63,24 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         return self.id2token[tokenid] # will throw for non-existent ids
 
 
+    def __iter__(self):
+        return iter(self.keys())
+
+
+    if PY3:
+        # restore Py2-style dict API
+        iterkeys = __iter__
+
+        def iteritems(self):
+            return self.items()
+
+        def itervalues(self):
+            return self.values()
+
+
     def keys(self):
         """Return a list of all token ids."""
-        return self.token2id.values()
+        return list(self.token2id.values())
 
 
     def __len__(self):
@@ -103,7 +123,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         """
         Convert `document` (a list of words) into the bag-of-words format = list
         of `(token_id, token_count)` 2-tuples. Each word is assumed to be a
-        **tokenized and normalized** utf-8 encoded string. No further preprocessing
+        **tokenized and normalized** string (either unicode or utf8-encoded). No further preprocessing
         is done on the words in `document`; apply tokenization, stemming etc. before
         calling this method.
 
@@ -117,8 +137,8 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         result = {}
         missing = {}
         if isinstance(document, string_types):
-            raise TypeError("doc2bow expects an array of utf8 tokens on input, not a string")
-        document = sorted(utils.to_utf8(token) for token in document)
+            raise TypeError("doc2bow expects an array of unicode tokens on input, not a single string")
+        document = sorted(utils.to_unicode(token) for token in document)
         # construct (word, frequency) mapping. in python3 this is done simply
         # using Counter(), but here i use itertools.groupby() for the job
         for word_norm, group in itertools.groupby(document):
@@ -171,7 +191,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
 
         # determine which tokens to keep
         good_ids = (v for v in itervalues(self.token2id)
-                      if no_below <= self.dfs[v] <= no_above_abs)
+                      if no_below <= self.dfs.get(v, 0) <= no_above_abs)
         good_ids = sorted(good_ids, key=self.dfs.get, reverse=True)
         if keep_n is not None:
             good_ids = good_ids[:keep_n]
@@ -220,28 +240,33 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         logger.debug("rebuilding dictionary, shrinking gaps")
 
         # build mapping from old id -> new id
-        idmap = dict(izip(itervalues(self.token2id),
-                     xrange(len(self.token2id))))
+        idmap = dict(izip(itervalues(self.token2id), xrange(len(self.token2id))))
 
         # reassign mappings to new ids
-        self.token2id = dict((token, idmap[tokenid])
-                             for token, tokenid in iteritems(self.token2id))
+        self.token2id = dict((token, idmap[tokenid]) for token, tokenid in iteritems(self.token2id))
         self.id2token = {}
-        self.dfs = dict((idmap[tokenid], freq)
-                        for tokenid, freq in iteritems(self.dfs))
+        self.dfs = dict((idmap[tokenid], freq) for tokenid, freq in iteritems(self.dfs))
 
 
-    def save_as_text(self, fname):
+    def save_as_text(self, fname, sort_by_word=True):
         """
         Save this Dictionary to a text file, in format:
-        `id[TAB]word_utf8[TAB]document frequency[NEWLINE]`.
+        `id[TAB]word_utf8[TAB]document frequency[NEWLINE]`. Sorted by word,
+        or by decreasing word frequency.
 
-        Note: use `save`/`load` to store in binary format instead (pickle).
+        Note: text format should be use for corpus inspection. Use `save`/`load`
+        to store in binary format (pickle) for improved performance.
         """
         logger.info("saving dictionary mapping to %s" % fname)
-        with utils.smart_open(fname, 'w') as fout:
-            for token, tokenid in sorted(iteritems(self.token2id)):
-                fout.write("%i\t%s\t%i\n" % (tokenid, token, self.dfs.get(tokenid, 0)))
+        with utils.smart_open(fname, 'wb') as fout:
+            if sort_by_word:
+                for token, tokenid in sorted(iteritems(self.token2id)):
+                    line = "%i\t%s\t%i\n" % (tokenid, token, self.dfs.get(tokenid, 0))
+                    fout.write(utils.to_utf8(line))
+            else:
+                for tokenid, freq in sorted(iteritems(self.dfs), key=lambda item: -item[1]):
+                    line = "%i\t%s\t%i\n" % (tokenid, self[tokenid], freq)
+                    fout.write(utils.to_utf8(line))
 
 
     def merge_with(self, other):
@@ -297,8 +322,9 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         Mirror function to `save_as_text`.
         """
         result = Dictionary()
-        with utils.smart_open(fname, 'rb') as f:
+        with utils.smart_open(fname) as f:
             for lineno, line in enumerate(f):
+                line = utils.to_unicode(line)
                 try:
                     wordid, word, docfreq = line[:-1].split('\t')
                 except Exception:
@@ -313,7 +339,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
 
 
     @staticmethod
-    def from_corpus(corpus):
+    def from_corpus(corpus, id2word=None):
         """
         Create Dictionary from an existing corpus. This can be useful if you only
         have a term-document BOW matrix (represented by `corpus`), but not the
@@ -321,8 +347,13 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
 
         This will scan the term-document count matrix for all word ids that
         appear in it, then construct and return Dictionary which maps each
-        `word_id -> str(word_id)`.
+        `word_id -> id2word[word_id]`.
+
+        `id2word` is an optional dictionary that maps the `word_id` to a token. In
+        case `id2word` isn't specified the mapping `id2word[word_id] = str(word_id)`
+        will be used.
         """
+
         result = Dictionary()
         max_id = -1
         for docno, document in enumerate(corpus):
@@ -334,10 +365,16 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
                 max_id = max(wordid, max_id)
                 result.num_pos += word_freq
                 result.dfs[wordid] = result.dfs.get(wordid, 0) + 1
-        # now make sure length(result) == get_max_id(corpus) + 1
-        for i in xrange(max_id + 1):
-            result.token2id[str(i)] = i
-            result.dfs[i] = result.dfs.get(i, 0)
+
+        if id2word is None:
+            # make sure length(result) == get_max_id(corpus) + 1
+            result.token2id = dict((unicode(i), i) for i in xrange(max_id + 1))
+        else:
+            # id=>word mapping given: simply copy it
+            result.token2id = dict((utils.to_unicode(token), id) for id, token in iteritems(id2word))
+        for id in itervalues(result.token2id):
+            # make sure all token ids have a valid `dfs` entry
+            result.dfs[id] = result.dfs.get(id, 0)
 
         logger.info("built %s from %i documents (total %i corpus positions)" %
                      (result, result.num_docs, result.num_pos))
