@@ -15,15 +15,15 @@ import logging
 
 from gensim import utils
 from gensim.corpora import IndexedCorpus
-from gensim._six import iteritems, iterkeys
-from gensim._six.moves import xrange, zip as izip
+from six import iteritems, iterkeys
+from six.moves import xrange, zip as izip
 
 
 logger = logging.getLogger('gensim.corpora.lowcorpus')
 
 
 def split_on_space(s):
-    return [word for word in s.strip().split(' ') if word]
+    return [word for word in utils.to_unicode(s).strip().split(' ') if word]
 
 
 class LowCorpus(IndexedCorpus):
@@ -91,13 +91,16 @@ class LowCorpus(IndexedCorpus):
     def _calculate_num_docs(self):
         # the first line in input data is the number of documents (integer). throws exception on bad input.
         with utils.smart_open(self.fname) as fin:
-            result = int(fin.readline())
+            try:
+                result = int(next(fin))
+            except StopIteration:
+                result = 0
+
         return result
 
     def __len__(self):
         return self.num_docs
 
-
     def line2doc(self, line):
         words = self.line2words(line)
 
@@ -127,7 +130,6 @@ class LowCorpus(IndexedCorpus):
         # note that this way, only one doc is stored in memory at a time, not the whole corpus
         return doc
 
-
     def __iter__(self):
         """
         Iterate over the corpus, returning one bag-of-words vector at a time.
@@ -137,7 +139,6 @@ class LowCorpus(IndexedCorpus):
                 if lineno > 0: # ignore the first line = number of documents
                     yield self.line2doc(line)
 
-
     @staticmethod
     def save_corpus(fname, corpus, id2word=None, metadata=False):
         """
@@ -153,16 +154,16 @@ class LowCorpus(IndexedCorpus):
         logger.info("storing corpus in List-Of-Words format into %s" % fname)
         truncated = 0
         offsets = []
-        with utils.smart_open(fname, 'w') as fout:
-            fout.write('%i\n' % len(corpus))
+        with utils.smart_open(fname, 'wb') as fout:
+            fout.write(utils.to_utf8('%i\n' % len(corpus)))
             for doc in corpus:
                 words = []
                 for wordid, value in doc:
                     if abs(int(value) - value) > 1e-6:
                         truncated += 1
-                    words.extend([str(id2word[wordid])] * int(value))
+                    words.extend([utils.to_unicode(id2word[wordid])] * int(value))
                 offsets.append(fout.tell())
-                fout.write('%s\n' % ' '.join(words))
+                fout.write(utils.to_utf8('%s\n' % ' '.join(words)))
 
         if truncated:
             logger.warning("List-of-words format can only save vectors with "
@@ -170,7 +171,6 @@ class LowCorpus(IndexedCorpus):
                             truncated)
         return offsets
 
-
     def docbyoffset(self, offset):
         """
         Return the document stored at file position `offset`.
@@ -178,4 +178,5 @@ class LowCorpus(IndexedCorpus):
         with utils.smart_open(self.fname) as f:
             f.seek(offset)
             return self.line2doc(f.readline())
-#endclass LowCorpus
+
+# endclass LowCorpus
