@@ -48,7 +48,7 @@ def read_doctopics(fname, eps=1e-6):
 
     """
     with utils.smart_open(fname) as fin:
-        fin.next()  # skip the header line
+        next(fin)  # skip the header line
         for lineno, line in enumerate(fin):
             parts = line.split()[2:]  # skip "doc" and "source" columns
             if len(parts) % 2 != 0:
@@ -136,7 +136,7 @@ class LdaMallet(utils.SaveLoad):
                     tokens = sum(([self.id2word[tokenid]] * int(cnt) for tokenid, cnt in doc), [])
                 else:
                     tokens = sum(([str(tokenid)] * int(cnt) for tokenid, cnt in doc), [])
-                fout.write("%s 0 %s\n" % (docno, utils.to_utf8(' '.join(tokens))))
+                fout.write(utils.to_utf8("%s 0 %s\n" % (docno, ' '.join(tokens))))
 
         # convert the text file above into MALLET's internal format
         cmd = self.mallet_path + " import-file --keep-sequence --remove-stopwords --token-regex '\S+' --input %s --output %s"
@@ -165,13 +165,16 @@ class LdaMallet(utils.SaveLoad):
     def __getitem__(self, bow, iterations=100):
         is_corpus, corpus = utils.is_corpus(bow)
         if not is_corpus:
+            # query is a single document => make a corpus out of it
             bow = [bow]
 
         self.convert_input(bow, infer=True)
         cmd = self.mallet_path + " infer-topics --input %s --inferencer %s --output-doc-topics %s --num-iterations %s"
         cmd = cmd % (self.fcorpusmallet() + '.infer', self.finferencer(), self.fdoctopics() + '.infer', iterations)
         logger.info("inferring topics with MALLET LDA '%s'" % cmd)
-        call(cmd, shell=True)
+        retval = call(cmd, shell=True)
+        if retval != 0:
+            raise RuntimeError("MALLET failed with error %s on return" % retval)
         result = list(read_doctopics(self.fdoctopics() + '.infer'))
         return result if is_corpus else result[0]
 
@@ -185,6 +188,7 @@ class LdaMallet(utils.SaveLoad):
             assert len(self.alpha) == self.num_topics, "mismatch between MALLET vs. requested topics"
             _ = next(fin)  # beta
             for lineno, line in enumerate(fin):
+                line = utils.to_unicode(line)
                 doc, source, pos, typeindex, token, topic = line.split()
                 tokenid = self.id2word.token2id[token] if hasattr(self.id2word, 'token2id') else int(token)
                 wordtopics[int(topic), tokenid] += 1
