@@ -80,7 +80,7 @@ class Dispatcher(object):
                     logger.info("registering worker #%i from %s" % (workerid, uri))
                     worker.initialize(workerid, dispatcher=self.callback, **model_params)
                     self.workers[workerid] = worker
-                except Pyro4.errors.PyroError, err:
+                except Pyro4.errors.PyroError:
                     logger.exception("unresponsive worker at %s, deleting it from the name server" % uri)
                     ns.remove(name)
 
@@ -183,7 +183,7 @@ def main():
     program = os.path.basename(sys.argv[0])
     # make sure we have enough cmd line parameters
     if len(sys.argv) < 1:
-        print globals()["__doc__"] % locals()
+        print(globals()["__doc__"] % locals())
         sys.exit(1)
 
     if len(sys.argv) < 2:
