@@ -12,11 +12,13 @@ import logging
 import os.path
 import unittest
 import tempfile
+import itertools
 
-from gensim.corpora import bleicorpus, mmcorpus, lowcorpus, svmlightcorpus, ucicorpus, malletcorpus
+from gensim.corpora import (bleicorpus, mmcorpus, lowcorpus, svmlightcorpus,
+                            ucicorpus, malletcorpus, textcorpus)
 
-
-module_path = os.path.dirname(__file__) # needed because sample data files are located in the same folder
+# needed because sample data files are located in the same folder
+module_path = os.path.dirname(__file__)
 datapath = lambda fname: os.path.join(module_path, 'test_data', fname)
 
 
@@ -25,18 +27,42 @@ def testfile():
     return os.path.join(tempfile.gettempdir(), 'gensim_corpus.tst')
 
 
-class CorpusTesterABC(object):
+class CorpusTestCase(unittest.TestCase):
     TEST_CORPUS = [[(1, 1.0)], [], [(0, 0.5), (2, 1.0)], []]
-    def __init__(self):
-        raise NotImplementedError("cannot instantiate Abstract Base Class")
-        self.corpus_class = None # to be overridden with a particular class
-        self.file_extension = None # file 'testcorpus.fileExtension' must exist and be in the format of corpusClass
+
+    def run(self, result=None):
+        if type(self) is not CorpusTestCase:
+            super(CorpusTestCase, self).run(result)
+
+    def tearDown(self):
+        # remove all temporary test files
+        fname = testfile()
+        extensions = ['', '', '.bz2', '.gz', '.index', '.vocab']
+        for ext in itertools.permutations(extensions, 2):
+            try:
+                os.remove(fname + ext[0] + ext[1])
+            except OSError:
+                pass
 
     def test_load(self):
         fname = datapath('testcorpus.' + self.file_extension.lstrip('.'))
         corpus = self.corpus_class(fname)
         docs = list(corpus)
-        self.assertEqual(len(docs), 9) # the deerwester corpus always has nine documents, no matter what format
+        # the deerwester corpus always has nine documents
+        self.assertEqual(len(docs), 9)
+
+    def test_empty_input(self):
+        with open(testfile(), 'w') as f:
+            f.write('')
+
+        with open(testfile() + '.vocab', 'w') as f:
+            f.write('')
+
+        corpus = self.corpus_class(testfile())
+        self.assertEqual(len(corpus), 0)
+
+        docs = list(corpus)
+        self.assertEqual(len(docs), 0)
 
     def test_save(self):
         corpus = self.TEST_CORPUS
@@ -48,9 +74,6 @@ class CorpusTesterABC(object):
         corpus2 = list(self.corpus_class(testfile()))
         self.assertEqual(corpus, corpus2)
 
-        # delete the temporary file
-        os.remove(testfile())
-
     def test_serialize(self):
         corpus = self.TEST_CORPUS
 
@@ -65,9 +88,6 @@ class CorpusTesterABC(object):
         for i in range(len(corpus)):
             self.assertEqual(corpus[i], corpus2[i])
 
-        # delete the temporary file
-        os.remove(testfile())
-
     def test_serialize_compressed(self):
         corpus = self.TEST_CORPUS
 
@@ -84,12 +104,10 @@ class CorpusTesterABC(object):
             for i in range(len(corpus)):
                 self.assertEqual(corpus[i], corpus2[i])
 
-            # delete the temporary file
-            os.remove(fname)
-#endclass CorpusTesterABC
+# endclass CorpusTestCase
 
 
-class TestMmCorpus(unittest.TestCase, CorpusTesterABC):
+class TestMmCorpus(CorpusTestCase):
     def setUp(self):
         self.corpus_class = mmcorpus.MmCorpus
         self.file_extension = '.mm'
@@ -97,33 +115,37 @@ class TestMmCorpus(unittest.TestCase, CorpusTesterABC):
     def test_serialize_compressed(self):
         # MmCorpus needs file write with seek => doesn't support compressed output (only input)
         pass
-#endclass TestMmCorpus
+
+# endclass TestMmCorpus
 
 
-class TestSvmLightCorpus(unittest.TestCase, CorpusTesterABC):
+class TestSvmLightCorpus(CorpusTestCase):
     def setUp(self):
         self.corpus_class = svmlightcorpus.SvmLightCorpus
         self.file_extension = '.svmlight'
-#endclass TestSvmLightCorpus
+
+# endclass TestSvmLightCorpus
 
 
-class TestBleiCorpus(unittest.TestCase, CorpusTesterABC):
+class TestBleiCorpus(CorpusTestCase):
     def setUp(self):
         self.corpus_class = bleicorpus.BleiCorpus
         self.file_extension = '.blei'
-#endclass TestBleiCorpus
+
+# endclass TestBleiCorpus
 
 
-class TestLowCorpus(unittest.TestCase, CorpusTesterABC):
+class TestLowCorpus(CorpusTestCase):
     TEST_CORPUS = [[(1, 1)], [], [(0, 2), (2, 1)], []]
 
     def setUp(self):
         self.corpus_class = lowcorpus.LowCorpus
         self.file_extension = '.low'
-#endclass TestLowCorpus
+
+# endclass TestLowCorpus
 
 
-class TestUciCorpus(unittest.TestCase, CorpusTesterABC):
+class TestUciCorpus(CorpusTestCase):
     TEST_CORPUS = [[(1, 1)], [], [(0, 2), (2, 1)], []]
 
     def setUp(self):
@@ -133,10 +155,11 @@ class TestUciCorpus(unittest.TestCase, CorpusTesterABC):
     def test_serialize_compressed(self):
         # UciCorpus needs file write with seek => doesn't support compressed output (only input)
         pass
-#endclass TestUciCorpus
+
+# endclass TestUciCorpus
 
 
-class TestMalletCorpus(unittest.TestCase, CorpusTesterABC):
+class TestMalletCorpus(CorpusTestCase):
     TEST_CORPUS = [[(1, 1)], [], [(0, 2), (2, 1)], []]
 
     def setUp(self):
@@ -147,14 +170,48 @@ class TestMalletCorpus(unittest.TestCase, CorpusTesterABC):
         fname = datapath('testcorpus.' + self.file_extension.lstrip('.'))
         corpus = self.corpus_class(fname)
         corpus.metadata = True
+        self.assertEqual(len(corpus), 9)
+
         docs = list(corpus)
-        self.assertEqual(len(docs), 9) # the deerwester corpus always has nine documents, no matter what format
+        self.assertEqual(len(docs), 9)
+
         for i, docmeta in enumerate(docs):
             doc, metadata = docmeta
-
             self.assertEqual(metadata[0], str(i + 1))
             self.assertEqual(metadata[1], 'en')
-#endclass TestMalletCorpus
+
+# endclass TestMalletCorpus
+
+
+class TestTextCorpus(CorpusTestCase):
+
+    def setUp(self):
+        self.corpus_class = textcorpus.TextCorpus
+        self.file_extension = '.txt'
+
+    def test_load_with_metadata(self):
+        fname = datapath('testcorpus.' + self.file_extension.lstrip('.'))
+        corpus = self.corpus_class(fname)
+        corpus.metadata = True
+        self.assertEqual(len(corpus), 9)
+
+        docs = list(corpus)
+        self.assertEqual(len(docs), 9)
+
+        for i, docmeta in enumerate(docs):
+            doc, metadata = docmeta
+            self.assertEqual(metadata[0], i)
+
+    def test_save(self):
+        pass
+
+    def test_serialize(self):
+        pass
+
+    def test_serialize_compressed(self):
+        pass
+
+# endclass TestTextCorpus
 
 
 if __name__ == '__main__':
